@@ -1,4 +1,4 @@
-## $Id: lmList.R,v 1.1 2000/07/03 18:22:44 bates Exp $
+## $Id: lmList.R,v 1.2.2.2 2000/12/07 19:12:35 bates Exp $
 ###
 ###                  Create a list of lm objects
 ###
@@ -9,13 +9,13 @@
 ### It is made available under the terms of the GNU General Public
 ### License, version 2, or at your option, any later version,
 ### incorporated herein by reference.
-### 
+###
 ### This program is distributed in the hope that it will be
 ### useful, but WITHOUT ANY WARRANTY; without even the implied
 ### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 ### PURPOSE.  See the GNU General Public License for more
 ### details.
-### 
+###
 ### You should have received a copy of the GNU General Public
 ### License along with this program; if not, write to the Free
 ### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
@@ -23,26 +23,28 @@
 
 ###*# Constructors
 
-lmList <- 
+lmList <-
   ## a list of lm objects from a formula or a groupedData object
   function(object, data, level, subset, na.action = na.fail, pool = TRUE)
   UseMethod("lmList")
 
-### FIXME: match.call problem
 lmList.groupedData <-
   function(object, data, level, subset, na.action = na.fail, pool = TRUE)
 {
   ### object will provide the formula, the data, and the groups
   form <- formula(object)
-  args <- match.call()
+  args <- as.list(match.call())[-1]
   args[["object"]] <- as.vector(eval(parse(text = paste(deparse(form[[2]]),
                                              "~", deparse(form[[3]][[2]])))))
-  args[["data"]] <- substitute(object)
-  args[[1]] <- as.name("lmList.formula")
-  eval(args, parent.frame(1))
+  if (!missing(data)) {
+    args[["data"]] <- substitute(object)
+  } else {
+    args <- as.list(c(args, list(data = substitute(object))))
+  }
+  do.call("lmList.formula", args)
 }
 
-lmList.formula <- 
+lmList.formula <-
   function(object, data, level, subset, na.action = na.fail, pool = TRUE)
 {
   Call <- match.call()
@@ -64,7 +66,7 @@ lmList.formula <-
       Call$object <-
         as.vector(eval(parse(text=paste(deparse(as.vector(Call$object)),
                                deparse(grpForm[[2]]), sep = "|"))))
-      
+
     } else {
       stop ("data must be a groupedData object if groups argument is missing")
     }
@@ -99,12 +101,12 @@ lmList.formula <-
   attr(val, "pool") <- pool
   class(val) <- "lmList"
   val
-}  
+}
 
 ###*# Methods for standard generics
 
-augPred.lmList <- 
-  function(object, primary = NULL, minimum = min(primary), 
+augPred.lmList <-
+  function(object, primary = NULL, minimum = min(primary),
 	   maximum = max(primary), length.out = 51, ...)
 {
   data <- eval(attr(object, "call")[["data"]])
@@ -147,7 +149,7 @@ augPred.lmList <-
   value <- rbind(orig, newvals)
   attributes(value[, 2]) <- attributes(groups)
   value[, ".type"] <- ordered(c(rep("original", nrow(data)),
-				rep("predicted", nrow(newvals))), 
+				rep("predicted", nrow(newvals))),
 			      levels = c("predicted", "original"))
   labs <- list(x = prName, y = respName)
   unts <- list(x = "", y = "")
@@ -155,10 +157,10 @@ augPred.lmList <-
     labs[names(attr(data, "labels"))] <- attr(data, "labels")
     unts[names(attr(data, "units"))] <- attr(data, "units")
     attr(value, "units") <- attr(data, "units")
-  } 
+  }
   attr(value, "labels") <- labs
   attr(value, "units") <- unts
-  attr(value, "formula") <- 
+  attr(value, "formula") <-
       eval(parse(text = paste(respName, "~", prName, "|", grName)))
   class(value) <- c("augPred", class(value))
   value
@@ -216,7 +218,7 @@ coef.lmList <-
   coefs
 }
 
-fitted.lmList <- 
+fitted.lmList <-
   function(object, subset = NULL, asList = FALSE)
 {
   if(!is.null(subset)) {
@@ -355,7 +357,7 @@ logLik.lmList <-
 		     sum(log(abs(diag(el$R)[1:p]))))
 		 }), 1, sum)
     N <- aux[1] - REML * aux[3]
-    val <- (aux[4] - N * (log(2 * pi) + 1 - log(N) + log(aux[2])))/2 - 
+    val <- (aux[4] - N * (log(2 * pi) + 1 - log(N) + log(aux[2])))/2 -
       REML * aux[5]
     attr(val, "nall") <- aux[1]
     attr(val, "nobs") <- N
@@ -370,8 +372,8 @@ logLik.lmList <-
   val
 }
 
-pairs.lmList <- 
-  function(object, form = ~ coef(.), label, id = NULL, idLabels = NULL, 
+pairs.lmList <-
+  function(object, form = ~ coef(.), label, id = NULL, idLabels = NULL,
 	   grid = FALSE, ...)
 {
   ## scatter plot matrix plots, generally based on coef or random.effects
@@ -381,7 +383,7 @@ pairs.lmList <-
   if (length(form) != 2) {
     stop("\"Form\" must be a one-sided formula")
   }
-  ## constructing data 
+  ## constructing data
   allV <- all.vars(asOneFormula(form, id, idLabels))
   allV <- allV[is.na(match(allV,c("T","F","TRUE","FALSE")))]
   if (length(allV) > 0) {
@@ -438,7 +440,7 @@ pairs.lmList <-
     argForm <- ~ .x
     argData <- list(.x = .x)
   }
-  
+
   auxData <- list()
   ## groups - need not be present
   grpsF <- getGroupsFormula(form)
@@ -451,11 +453,11 @@ pairs.lmList <-
       argForm <- eval(parse(text = paste("~ .x |", deparse(grpsF[[2]]))))
     else argForm <- eval(parse(text = paste(".y ~ .x |", deparse(grpsF[[2]]))))
   }
-  
+
   ## id and idLabels - need not be present
   if (!is.null(id)) {			# identify points in plot
     N <- attr(object, "dims")$N
-    id <- 
+    id <-
       switch(mode(id),
 	     numeric = {
 	       if ((id <= 0) || (id >= 1)) {
@@ -484,7 +486,7 @@ pairs.lmList <-
       } else if (is.vector(idLabels)) {
 	if (length(idLabels <- unlist(idLabels)) != N) {
 	  stop("\"IdLabels\" of incorrect length")
-	} 
+	}
 	idLabels <- as.character(idLabels)
       } else {
 	stop("\"IdLabels\" can only be a formula or a vector")
@@ -497,7 +499,7 @@ pairs.lmList <-
   }
 
   if (length(auxData)) {		# need collapsing
-    auxData <- gsummary(data.frame(auxData), 
+    auxData <- gsummary(data.frame(auxData),
 			groups = getGroups(object))
     auxData <- auxData[row.names(.x), , drop = FALSE]
     if (!is.null(auxData[[".g"]])) {
@@ -514,7 +516,7 @@ pairs.lmList <-
     wchDat <- is.na(match(names(auxData), c(".id", ".idLabels")))
     if (any(wchDat)) {
       argData <- cbind(argData, auxData[, wchDat, drop = FALSE])
-      
+
     }
   }
 
@@ -533,8 +535,8 @@ pairs.lmList <-
   ## defining the type of plot
   if (length(argForm) == 3) {		# xyplot
     plotFun <- "xyplot"
-    args <- c(args, 
-	      panel = list(function(x, y, subscripts, ...) 
+    args <- c(args,
+	      panel = list(function(x, y, subscripts, ...)
 		  {
                     dots <- list(...)
 		    if (grid) panel.grid()
@@ -547,7 +549,7 @@ pairs.lmList <-
 		  }))
   } else {				# splom
     plotFun <- "splom"
-    args <- c(args, 
+    args <- c(args,
 	      panel = list(function(x, y, subscripts, ...)
 		  {
                     dots <- list(...)
@@ -601,7 +603,7 @@ plot.intervals.lmList <-
 	    xx <- x[ok]
 	    dot.symbol <- trellis.par.get("dot.symbol")
 	    dot.line <- trellis.par.get("dot.line")
-	    abline(h = yy, lwd = dot.line$lwd, lty = dot.line$lty, col = 
+	    abline(h = yy, lwd = dot.line$lwd, lty = dot.line$lty, col =
 		   dot.line$col)
 	    points(xx, yy, pch = "|", col = col, cex = cex, font = font, ...)
 	    lower <- tapply(xx, yy, min)
@@ -617,23 +619,22 @@ plot.intervals.lmList <-
 }
 
 plot.ranef.lmList <-
-  function(object, form = NULL, grid = TRUE, control, ...)   
+  function(object, form = NULL, grid = TRUE, control, ...)
 {
-  fArgs <- match.call()
-  fArgs[[1]] <- as.name("plot.ranef.lme")
-  eval(fArgs, envir = parent.frame(1))
+  fArgs <- as.list(match.call())[-1]
+  do.call("plot.ranef.lme", fArgs)
 }
 
-plot.lmList <- 
-  function(object, form = resid(., type = "pool") ~ fitted(.), abline, 
-	   id = NULL, idLabels = NULL,  grid, ...)  
+plot.lmList <-
+  function(object, form = resid(., type = "pool") ~ fitted(.), abline,
+	   id = NULL, idLabels = NULL,  grid, ...)
   ## Diagnostic plots based on residuals and/or fitted values
 {
   if (!inherits(form, "formula")) {
     stop("\"Form\" must be a formula")
   }
 
-  ## constructing data 
+  ## constructing data
   allV <- all.vars(asOneFormula(form, id, idLabels))
   allV <- allV[is.na(match(allV,c("T","F","TRUE","FALSE")))]
   if (length(allV) > 0) {
@@ -684,7 +685,7 @@ plot.lmList <-
     else if (!is.null(rF) && (xlab == rF)) xlab <- rL
   }
   if (is.null(args$xlab)) args$xlab <- xlab
-      
+
   ## response - need not be present
   respF <- getResponseFormula(form)
   if (!is.null(respF)) {
@@ -719,7 +720,7 @@ plot.lmList <-
   if (is.null(args$adj)) args$adj <- par("adj")
 
   if (!is.null(id)) {			# identify points in plot
-    id <- 
+    id <-
       switch(mode(id),
 	     numeric = {
 	       if ((id <= 0) || (id >= 1)) {
@@ -741,7 +742,7 @@ plot.lmList <-
       } else if (is.vector(idLabels)) {
 	if (length(idLabels <- unlist(idLabels)) != length(id)) {
 	  stop("\"IdLabels\" of incorrect length")
-	} 
+	}
 	idLabels <- as.character(idLabels)
       } else {
 	stop("\"IdLabels\" can only be a formula or a vector")
@@ -766,8 +767,8 @@ plot.lmList <-
   if (length(argForm) == 3) {
     if (is.numeric(.y)) {		# xyplot
       plotFun <- "xyplot"
-      args <- c(args, 
-		panel = list(function(x, y, subscripts, ...) 
+      args <- c(args,
+		panel = list(function(x, y, subscripts, ...)
 		    {
                       dots <- list(...)
 		      if (grid) panel.grid()
@@ -783,8 +784,8 @@ plot.lmList <-
 		    }))
     } else {				# assume factor or character
       plotFun <- "bwplot"
-      args <- c(args, 
-		panel = list(function(x, y, ...) 
+      args <- c(args,
+		panel = list(function(x, y, ...)
 		    {
 		      if (grid) panel.grid()
 		      panel.bwplot(x, y)
@@ -795,8 +796,8 @@ plot.lmList <-
     }
   } else {
     plotFun <- "histogram"
-    args <- c(args, 
-	      panel = list(function(x, y, ...) 
+    args <- c(args,
+	      panel = list(function(x, y, ...)
 		  {
 		    if (grid) panel.grid()
 		    panel.histogram(x, y)
@@ -815,8 +816,8 @@ plot.lmList <-
   do.call(plotFun, args)
 }
 
-predict.lmList <- 
-  function(object, newdata, subset = NULL, pool = attr(object, "pool"), 
+predict.lmList <-
+  function(object, newdata, subset = NULL, pool = attr(object, "pool"),
 	   asList = FALSE, se.fit = FALSE)
 {
   if(missing(newdata)) {
@@ -833,7 +834,7 @@ predict.lmList <-
     formGrps <- getGroupsFormula(object)
     if(all(match(all.vars(formGrps), names(newdata), 0))) {
       ## newdata contains groups definition
-      grps <- getGroups(newdata, getGroupsFormula(object, asList = TRUE), 
+      grps <- getGroups(newdata, getGroupsFormula(object, asList = TRUE),
 			level = attr(object, "level"))
       grps <- pruneLevels(grps)
       subset <- as.character(unique(grps))
@@ -844,7 +845,7 @@ predict.lmList <-
       newdata <- NULL
       sameData <- FALSE
     }
-  }				       
+  }
   if(!is.null(subset)) {
     if(any(is.na(match(subset, names(object))))) {
       stop("Non-existent group requested in \"subset\".")
@@ -884,8 +885,8 @@ predict.lmList <-
   } else {
     if(pool) {
       poolSD <- pooledSD(object)
-      val[noNull] <- 
-	lapply(object[noNull], 
+      val[noNull] <-
+	lapply(object[noNull],
 	       function(el, newdata, se.fit, poolSD) {
 		 aux <- predict(el, newdata, se.fit = se.fit)
 		 if(se.fit) {
@@ -896,8 +897,8 @@ predict.lmList <-
 		 }
 	       }, newdata = newdata, se.fit = se.fit, poolSD = poolSD)
     } else {
-      val[noNull] <- 
-	lapply(object[noNull], 
+      val[noNull] <-
+	lapply(object[noNull],
 	       function(el, newdata, se.fit) {
 		 aux <- predict(el, newdata, se.fit = se.fit)
 		 if(se.fit) {
@@ -950,8 +951,8 @@ print.intervals.lmList <-
   attr(x, "groupsName") <- NULL
   print(x, ...)
 }
-  
-print.lmList <- 
+
+print.lmList <-
   function(x, pool = attr(x, "pool"), ...)
 {
   mCall <- attr(x, "call")
@@ -1006,12 +1007,12 @@ print.summary.lmList <-
 qqnorm.lmList <-
   function(object, form = ~ resid(., type = "pooled"), abline = NULL,
            id = NULL, idLabels = NULL, grid = FALSE, resType = "pool", ...)
-  ## normal probability plots for residuals and random effects 
+  ## normal probability plots for residuals and random effects
 {
   if (!inherits(form, "formula")) {
     stop("\"Form\" must be a formula")
   }
-  ## constructing data 
+  ## constructing data
   allV <- all.vars(asOneFormula(form, id, idLabels))
   allV <- allV[is.na(match(allV,c("T","F","TRUE","FALSE")))]
   if (length(allV) > 0) {
@@ -1034,7 +1035,7 @@ qqnorm.lmList <-
   else args <- list()
   ## appending object to data
   data <- as.list(c(as.list(data), . = list(object)))
-  
+
   ## covariate - must always be present
   covF <- getCovariateFormula(form)
   .x <- eval(covF[[2]], data)
@@ -1060,7 +1061,7 @@ qqnorm.lmList <-
       dform <- paste(dform, deparse(grp[[2]]), sep = "|")
     }
     if (!is.null(id)) {			# identify points in plot
-      id <- 
+      id <-
         switch(mode(id),
                numeric = {
                  if ((id <= 0) || (id >= 1)) {
@@ -1083,7 +1084,7 @@ qqnorm.lmList <-
         } else if (is.vector(idLabels)) {
           if (length(idLabels <- unlist(idLabels)) != length(id)) {
             stop("\"IdLabels\" of incorrect length")
-          } 
+          }
           idLabels <- as.character(idLabels)
         } else {
           stop("\"IdLabels\" can only be a formula or a vector")
@@ -1113,7 +1114,7 @@ qqnorm.lmList <-
     ## id and idLabels - need not be present
     if (!is.null(id)) {			# identify points in plot
       N <- object$dims$N
-      id <- 
+      id <-
         switch(mode(id),
                numeric = {
                  if ((id <= 0) || (id >= 1)) {
@@ -1129,7 +1130,7 @@ qqnorm.lmList <-
         ## id as a formula evaluated in data
         auxData[[".id"]] <- id
       }
-      
+
       if (is.null(idLabels)) {
         idLabels <- rep(row.names(.x), nc)
       } else {
@@ -1139,7 +1140,7 @@ qqnorm.lmList <-
         } else if (is.vector(idLabels)) {
           if (length(idLabels <- unlist(idLabels)) != N) {
             stop("\"IdLabels\" of incorrect length")
-          } 
+          }
           idLabels <- as.character(idLabels)
         } else {
           stop("\"IdLabels\" can only be a formula or a vector")
@@ -1152,14 +1153,14 @@ qqnorm.lmList <-
     }
 
     if (length(auxData)) {		# need collapsing
-      auxData <- gsummary(data.frame(auxData), 
+      auxData <- gsummary(data.frame(auxData),
                           groups = getGroups(object, level = level))
       auxData <- auxData[row.names(.x), , drop = FALSE]
 
       if (!is.null(auxData[[".id"]])) {
         id <- rep(auxData[[".id"]], nc)
       }
-      
+
       if (!is.null(auxData[[".Lid"]])) {
         idLabels <- rep(auxData[[".Lid"]], nc)
       }
@@ -1197,7 +1198,7 @@ qqnorm.lmList <-
   }
   do.call("xyplot", args)
 }
-  
+
 ranef.lmList <-
   ##  Extracts the random effects from an lmList object.
   ##  If aug.frame is true, the returned data frame is augmented with a
@@ -1213,7 +1214,7 @@ ranef.lmList <-
   effs <- val[, effNames, drop = FALSE]
   effs <-
     as.data.frame(lapply(effs, function(el) el - mean(el, na.rm = TRUE)))
-  
+
   if(standard) {
     stdEff <- unlist(lapply(effs, function(el) sqrt(var(el[!is.na(el)]))))
     effs <- as.data.frame(as.matrix(effs) %*% diag(1/stdEff))
@@ -1227,7 +1228,7 @@ ranef.lmList <-
   val
 }
 
-residuals.lmList <- 
+residuals.lmList <-
   function(object, type = c("response", "pearson", "pooled.pearson"),
 	   subset = NULL, asList = FALSE)
 {
@@ -1255,7 +1256,7 @@ residuals.lmList <-
     attr(object, "call") <- oatt
     class(object) <- oclass
   }
-  val <- 
+  val <-
     switch(type,
 	   pooled.pearson = {
 	     lapply(object, function(el, pSD) {
@@ -1297,7 +1298,7 @@ residuals.lmList <-
   attr(val, "label") <- lab
   val
 }
-	     
+
 summary.lmList <-
   function(object, pool = attr(object, "pool"))
 {
@@ -1315,7 +1316,7 @@ summary.lmList <-
 			   c(dimnames(template), list(names(lst)))),
 		     c(3, 2, 1))
 	val[unlist(lapply(lst, is.null)), , ] <- NA
-	val	
+	val
       }
   to.2d.array <-
     ## Convert the list to a 2d array watching for null elements
@@ -1335,7 +1336,7 @@ summary.lmList <-
       }
   ## Create a summary by applying summary to each component of the list
   sum.lst <- lapply(object, function(el) if(is.null(el)) {NULL}
-                                         else {summary(el)})  
+                                         else {summary(el)})
   nonNull <- !unlist(lapply(sum.lst, is.null))
   if(!any(nonNull)) {
     return(NULL)
@@ -1406,7 +1407,7 @@ update.lmList <-
     nextCall$object <- update(as.formula(nextCall$object), nextCall$object)
   }
   nextCall[[1]] <- as.name("lmList")
-  do.call(nextCall, envir = parent.frame(1))
+  eval(nextCall, envir = parent.frame(1))
 }
 
 ### Local variables:
